/*
 *  ngtk/c/ngtk_readline.c
 *
 *  Copyright (C) 1998, 1999 Gabriel Fort <gabx@tfz.net>
 *
 */

/*----------------------------------------------------------------------------

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the 
    Free Software Foundation, Inc., 59 Temple Place - Suite 330, 
    Boston, MA  02111-1307  USA.

----------------------------------------------------------------------------*/
#include <ngtk/ngtk.h>

void ngtk_virt_readline(WINDOW *win, int vislen, int virtlen, char *str)
{
    int virt_locate = strlen(str);
	int vis_locate;
    int buf = 0;
	int insert = 1;
	int offset = 0;

	memset(&str[virt_locate], 32, virtlen-virt_locate);

	if (virt_locate <= vislen)
		vis_locate = virt_locate;
	else
        vis_locate = vislen;

	if (virt_locate >= vislen)
		offset = virt_locate-vislen+1;
	else
		offset = 0;

	mvwprintw(win, 0, 0, "%s", &str[offset]);
	wmove(win, 0, vis_locate);
	wrefresh(win);

    while (1) {
		buf = ngtk_get_keysym();

        switch (buf) {
            case CR:
				str[ngtk_strpad(str, virtlen)+1] = 0;
                return;

            case ESC:
                strcpy(str, __ERR__);
                return;

            case NGTK_KEY_BACKSPACE:
                if (!virt_locate)
                    continue;
                virt_locate--;
				ngtk_delchar(str, virtlen, virt_locate);
                break;

			case NGTK_KEY_DELETE:
				ngtk_delchar(str, virtlen, virt_locate);
				break;

            case NGTK_KEY_LEFT:
				if (!virt_locate)
					continue;
                virt_locate--;
                break;

            case NGTK_KEY_RIGHT:
				if (virt_locate+1 >= virtlen)
					break;
                virt_locate++;
                break;

            case NGTK_KEY_HOME:
                virt_locate = 0;
                break;

			case NGTK_KEY_END:
				virt_locate = ngtk_strpad(str, virtlen)+1;
				break;

			case NGTK_KEY_IC:
                insert = !insert;
				break;

            default:
                if (buf >= 0x100)
                    break;
                if (virt_locate >= virtlen)
                    break;

				if (insert)
					ngtk_inschar(str, virtlen, buf, virt_locate);
				else
					str[virt_locate] = buf;

                virt_locate++;
        }

		if (virt_locate <= vislen)
			vis_locate = virt_locate;
		else
			vis_locate = vislen;

		if (virt_locate >= vislen)
			offset = virt_locate-vislen+1;
		else
			offset = 0;

		mvwprintw(win, 0, 0, "%s", &str[offset]);
        wmove(win, 0, vis_locate);
		wrefresh(win);
    }
}


void ngtk_readline(WINDOW *win, int maxlen, char *str)
{
	ngtk_virt_readline(win, maxlen, maxlen, str);
}


void ngtk_virt_read_win(int len, int virtlen, int top, int left, int cpair, int attr, char *str)
{
	ngtk_win_t *input_win;

	input_win = ngtk_push_win(1, len, top, left, cpair, attr, 1, 0);
    ngtk_clrwin(input_win->win, 32, 1);
	ngtk_virt_readline(input_win->win, len, virtlen, str);
    ngtk_pop_win(input_win);
}

void ngtk_read_win(int len, int top, int left, int cpair, int attr, char *str)
{
	ngtk_virt_read_win(len, len, top, left, cpair, attr, str);
}


